//
//  MCHelperToolClient.m
//  MemoryCell
//
//  Created by Quentin Carnicelli on 9/9/06.
//  Copyright 2006 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "MCHelperToolClient.h"
#import "MCHelperTool.h"

@interface MCHelperToolClient(Private)
- (BOOL)_connectToHelperToolIfNeeded;
- (void)_disconnectFromHelperTool;
@end

@implementation MCHelperToolClient

+ (BOOL)_hasDirectTFP
{
	int err;
	int val = 0;
	size_t sizeOfVal = sizeof(val);

	err = sysctlbyname("kern.tfp.policy", &val, &sizeOfVal, nil, nil );	
	
#if __ppc__
	if( err == -1 )
		return YES;
#endif
	if( err )
		return NO;

	return (val < 2);
}

+ (NSString*)helperToolPath
{
	return [[NSBundle bundleForClass: [self class]] pathForAuxiliaryExecutable: @"MemoryCellHelper"];
}

+ (BOOL)helperToolHasValidPermissions
{
	NSString* path = [self helperToolPath];
	NSDictionary* attrs;

	if( [self _hasDirectTFP] )
		return YES; //We don't care then
	
	if( !path )
		return NO;
		
	attrs = [[NSFileManager defaultManager] fileAttributesAtPath: path traverseLink: YES];
	if( !attrs )
		return NO;

	if( ![attrs fileGroupOwnerAccountName] || [[attrs fileGroupOwnerAccountName] isEqual: @MCHelperToolFileGroupOwner] == NO )
		return NO;

	if( [attrs filePosixPermissions] != MCHelperToolFilePermissions )
		return NO;

	return YES;
}

- (id)init
{
	if( (self == [super init]) != nil )
	{
		mUseRemoteTFP = ([[self class] _hasDirectTFP] == NO);
	}
	
	return self;
}

- (void)dealloc
{
	[self _disconnectFromHelperTool];
	[super dealloc];
}

- (id <MCHelperToolInterface>)_helperToolProxy
{
	return (id <MCHelperToolInterface>)[mConnection rootProxy];
}

- (void)_disconnectFromHelperTool
{
	@try
	{
		[[self _helperToolProxy] terminate];
		[mConnection release];
		mConnection = nil;
	}
	@catch( NSException* localException )
	{
		NSLog( @"MemoryCell Error: %@", localException );
		mConnection = nil;
	}

	@try
	{
		[mTask terminate];
		[mTask release];
		mTask = nil;
	}
	@catch( NSException* localException )
	{
		NSLog( @"MemoryCell Error: %@", localException );
		mTask = nil;
	}
}

- (BOOL)_connectToHelperToolIfNeeded
{
	NSString* err = @"";

	if( mConnection && [mConnection isValid] == NO )
		[self _disconnectFromHelperTool];

	if( !mTask )
	{
		NSString* binPath;
		
		if( ![[self  class] helperToolHasValidPermissions] )
		{
			err = @"Invalid Permissions";
			goto fail;
		}

		binPath = [[self class] helperToolPath];
		if( !binPath )
		{
			err = @"Could not find MemoryCellHelper";
			goto fail;
		}

		mTask = [[NSTask launchedTaskWithLaunchPath: binPath arguments: [NSArray array]] retain];
		if( !mTask )
		{
			err = @"Could not launch MemoryCellHelper";
			goto fail;
		}
	}

	if( !mConnection )
	{
		NSString* portName;
		NSTimeInterval timeout;
		
		portName = [NSString stringWithFormat: MCHelperPortNameFormat, [mTask processIdentifier]];
	
		timeout = [NSDate timeIntervalSinceReferenceDate] + 1.0f;
	
		while( [NSDate timeIntervalSinceReferenceDate] < timeout )
		{
			mConnection = [[NSConnection connectionWithRegisteredName: portName host: nil] retain];
			if( mConnection )
				break;
		}
		
		if( !mConnection )
		{
			err = @"Could not connect to MemoryCellHelper";
			goto fail;
		}

		[mConnection setRequestTimeout: 0.05];
		[mConnection setReplyTimeout: 0.05];
	
		[NSThread sleepUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.25f]]; //This is lame
	}

	return YES;
fail:
	if( err )
		NSLog( @"MemoryCell Error: %@\n", err );

	return NO;
}


- (NSData*)local_loadProcessMemoryStatsForPid: (pid_t)pid compareStats: (NSData*)oldStatsData
{
	int err;
	process_memory_stats oldStats;
	process_memory_stats newStats;

	if( oldStatsData )
	{
		err = process_mem_stats_from_data( (CFDataRef)oldStatsData, &oldStats);
		if( !err )
			err = process_get_mem_stats( pid, &oldStats, &newStats );
	}
	else
	{
		err = process_get_mem_stats( pid, nil, &newStats );
	}

	if( err )
	{
		memset( &newStats, 0, sizeof(newStats) );
		newStats.fPid = pid;
		newStats.fStatus = kProcessMemoryStatsError;
	}

	return [NSData dataWithBytes: &newStats length: sizeof(newStats)];
}


- (NSData*)remote_loadProcessMemoryStatsForPid: (pid_t)pid compareStats: (NSData*)oldStats
{
	NSData* data = nil;
	
	@try
	{
		if( [self _connectToHelperToolIfNeeded] )
		{
			data = [[self _helperToolProxy] processMemoryStatsForPid: pid lastStats: oldStats];
		}
	}
	@catch( NSException* localException )
	{
		process_memory_stats newStats;

		memset( &newStats, 0, sizeof(newStats) );
		newStats.fPid = pid;
		newStats.fStatus = kProcessMemoryStatsError;

		data = [NSData dataWithBytes: &newStats length: sizeof(newStats)];
	}
	
	return data;
}

- (NSData*)loadProcessMemoryStatsForPid: (pid_t)pid compareStats: (NSData*)oldStats
{
	if( mUseRemoteTFP )
		return [self remote_loadProcessMemoryStatsForPid: pid compareStats: oldStats];

	return [self local_loadProcessMemoryStatsForPid: pid compareStats: oldStats];
	
}

@end
