//
//  MCMenuExtra.m
//  MemoryCell
//
//  Created by Quentin Carnicelli on 11/29/04.
//  Copyright 2004 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "MCMenuExtra.h"

#import "MCMenuExtraView.h"
#import "MCProcessMonitor.h"
#import "MCAboutBox.h"
#import "MCHelperToolClient.h"

#define _kAutomaticRefreshInterval 10.0

@interface MCMenuExtra(MenuAdditions)
- (void)_setupMenu;
@end

@implementation MCMenuExtra

- (void)_setupModel
{
	mProcessMonitor = [[MCProcessMonitor alloc] init];

	[[NSNotificationCenter defaultCenter]
		addObserver: self
		selector: @selector(_processMonitorChanged:)
		name: MCProcessMonitorChangeNotification
		object: mProcessMonitor];
}

- (void)_setupView
{
	MCMenuExtraView* view;
	NSRect frame;
	
	frame.origin = NSZeroPoint;
	frame.size = [MCMenuExtraView contentSize];
	
	view = [[MCMenuExtraView alloc] initWithFrame: frame menuExtra: self];
	[self setView: view];
	[view release];
}

- (void)_loadPreferences
{
	NSString* bundleID			= [[NSBundle bundleForClass: [self class]] bundleIdentifier];
	NSUserDefaults* defaults	= [NSUserDefaults standardUserDefaults];
	NSDictionary* domain		= [defaults persistentDomainForName: bundleID];
	NSNumber* unitNum;
	process_memory_stat_unit unit;
	
	unitNum = [domain objectForKey: @"displayMemoryUnit"];
	if( unitNum != nil )
		unit = [unitNum intValue];
	else
		unit = kProcessPrivateMemorySize;
	
	[[self view] setUnit: unit];
}

- (void)_savePreferences
{
	NSString* bundleID			= [[NSBundle bundleForClass: [self class]] bundleIdentifier];
	NSUserDefaults* defaults	= [NSUserDefaults standardUserDefaults];
	NSMutableDictionary* domain	= [[defaults persistentDomainForName: bundleID] mutableCopy];
	NSNumber* unitNum;

	unitNum = [NSNumber numberWithInt: [[self view] unit]];
	[domain setObject: unitNum forKey: @"displayMemoryUnit"];
		
	[defaults setPersistentDomain: domain forName: bundleID];	
	
	[domain release];
}

- (void)_processMonitorChanged: (id)sender
{
	pid_t newPid = [mProcessMonitor foregroundProcess];
	
	[[self view] setProcess: [mProcessMonitor statsForProcess: newPid]];
}

- (id)initWithBundle:(NSBundle*)bundle
{
	self = [super initWithBundle: bundle];
	
	if( self )
	{
		[self _setupModel];
		[self _setupView];
		[self _setupMenu];
		[self _loadPreferences];
		[self _processMonitorChanged: self];
	}
	
	return self;
}

- (void)_safeUnload
{
	[[NSNotificationCenter defaultCenter] removeObserver: self];
	
	[mProcessMonitor release];
	mProcessMonitor = nil;
	
	[self _savePreferences];
}

- (void)willUnload
{
	[self _safeUnload];
	[super willUnload];
}

- (void)dealloc
{
	[self _safeUnload];
	[super dealloc];
}

- (NSString*)AXDescription
{
	return @"MemoryCell menu extra";
}

- (NSString*)AXValue
{
	return [(MCMenuExtraView*)[self view] AXValue];
}

@end

#pragma mark -

@implementation MCMenuExtra(MenuAdditions)

- (IBAction)changedDisplayUnit: (id)sender
{
	process_memory_stat_unit unit;

	if( ![sender respondsToSelector: @selector( tag )] )
		return;

	unit = [sender tag];
	if( unit > 0 )
		[[self view] setUnit: unit];
}

- (IBAction)hitOpenAbout: (id)sender
{
	[[MCAboutBox sharedAboutBox] showWindow: self];
}

- (IBAction)hitOpenActMonitor: (id)sender
{
	[[NSWorkspace sharedWorkspace] launchAppWithBundleIdentifier: @"com.apple.ActivityMonitor"
								   options: NSWorkspaceLaunchDefault
								   additionalEventParamDescriptor: nil
								   launchIdentifier: nil];
}

- (NSString*)longFormatMemorySize: (unsigned long)inSize change: (long)change
{
	float size;
	NSString* trend;
	NSString* unit;

	if( inSize <= 0)
		return @"(N/A)";

	if( abs( change ) < (50*1024) )
		trend = @"";
	else if( change < 0 )
		trend = @"-";
	else
		trend = @"+";

	size = (float)inSize / 1024.0;
	if( size < 1000 )
		unit = @"KB";
	else
	{

		size /= 1024.0;
		if( size < 1000 )
			unit = @"MB";
		else
		{
			size /= 1024.0;
			unit = @"GB";
		}
	}

	//return [NSString stringWithFormat: @"%.1f %@ %@", size, unit, trend];
	return [NSString stringWithFormat: @"%.1f %@", size, unit];
}

- (NSString*)_displayNameForProcess: (const process_memory_stats*)process
{
	pid_t pid = process->fPid;
	ProcessSerialNumber psn;
	NSString* name;
	
	if( GetProcessForPID( pid,  &psn ) == noErr )
		if( CopyProcessName( &psn, (CFStringRef*)&name ) == noErr )
			return [name autorelease];
			
	return [NSString stringWithFormat: @"(Unknown #%d)", pid];
}

- (void)menuNeedsUpdate:(NSMenu*)menu
{
	NSEnumerator* itemEnum = [[menu itemArray] objectEnumerator];
	NSMenuItem* menuItem;
	const process_memory_stats* process;
	int tag;
	process_memory_stat_unit unit;
	NSString* formatStr;
	unsigned long size;
	long change;
	NSString* title;

	process = [[self view] process];
	if( !process || !process->fPid || process->fStatus == kProcessMemoryStatsError )
	{
		NSMutableArray* removeItems = [NSMutableArray array];
		NSString* errStr;
		
		while( (menuItem = [itemEnum nextObject]) != nil )
		{
			if( [menuItem tag] > 0 )
				[removeItems addObject: menuItem];
		}
		
		itemEnum = [removeItems objectEnumerator];
		while( (menuItem = [itemEnum nextObject]) != nil )
			[menu removeItem: menuItem];
		
		if( [MCHelperToolClient helperToolHasValidPermissions] == NO )
		{
			errStr = @"MemoryCell has incorrect file permissions and must be re-installed.";
		}
		else
		{
			errStr = @"An unknown error has occured, check Console.";
		}
		
		menuItem = (NSMenuItem*)[menu itemWithTag: -1];
		[menuItem setTitle: errStr];

	}
	else
	{
		if( [[self menu] numberOfItems] < 10 )
			[self _setupMenu];
	
		//** Update the application name
		menuItem = (NSMenuItem*)[menu itemWithTag: -1];
		[menuItem setTitle: [NSString stringWithFormat: @"%@ (%d):", [self _displayNameForProcess: process], process->fPid]];

		//** Update the memory-unit based items
		while( (menuItem = [itemEnum nextObject]) != nil )
		{
			unit = [menuItem tag];
			if( unit <= 0 )
				continue;
		
			formatStr = [menuItem representedObject];
			if( formatStr == nil || ![formatStr isKindOfClass: [NSString class]] )
				continue;
			
			size = process_mem_stats_get_value( process, unit );
			change = process_mem_stats_get_value_change( process, unit );
			
			title = [self longFormatMemorySize: size change: change];
			title = [NSString stringWithFormat: formatStr, title];
			
			[menuItem setTitle: title];
			[menuItem setState: unit == [[self view] unit]];
		}
	}
}

- (void)_setupMenu
{
	NSMenu* menu = [[NSMenu alloc] initWithTitle: @"MemoryCell menu extra"];
	NSMenuItem* menuItem;
	
	[menu setDelegate: self];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"" action: nil keyEquivalent: @""];
	[menuItem setTag: -1];
	[menuItem setEnabled: NO];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"" action: @selector(changedDisplayUnit:) keyEquivalent: @""];
	[menuItem setTarget: self];
	[menuItem setRepresentedObject: @"    Private Memory: %@"];
	[menuItem setTag: kProcessPrivateMemorySize];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"" action: @selector(changedDisplayUnit:) keyEquivalent: @""];
	[menuItem setTarget: self];
	[menuItem setRepresentedObject: @"    Real Memory: %@"];
	[menuItem setTag: kProcessRealMemorySize];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"" action: @selector(changedDisplayUnit:) keyEquivalent: @""];
	[menuItem setTarget: self];
	[menuItem setRepresentedObject: @"    Shared Memory: %@"];
	[menuItem setTag: kProcessSharedMemorySize];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"" action: @selector(changedDisplayUnit:) keyEquivalent: @""];
	[menuItem setTarget: self];
	[menuItem setRepresentedObject: @"    Virtual Private Memory: %@"];
	[menuItem setTag: kProcessVirtualPrivateMemorySize];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"" action: @selector(changedDisplayUnit:) keyEquivalent: @""];
	[menuItem setTarget: self];
	[menuItem setRepresentedObject: @"    Virtual Memory: %@"];
	[menuItem setTag: kProcessVirtualMemorySize];

	[menu addItem: [NSMenuItem separatorItem]];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"Open Activity Monitor..." action: @selector(hitOpenActMonitor:) keyEquivalent: @""];
	[menuItem setTarget: self];

	[menu addItem: [NSMenuItem separatorItem]];

	menuItem = (NSMenuItem*)[menu addItemWithTitle: @"About MemoryCell" action: @selector(hitOpenAbout:) keyEquivalent: @""];
	[menuItem setTarget: self];

	[self setMenu: menu];
	[menu release];
}

@end
