//
//  MCProcessMonitor.m
//  MemoryCell
//
//  Created by Quentin Carnicelli on 9/10/06.
//  Copyright 2006 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "MCProcessMonitor.h"
#import "MCForegroundProcessMonitor.h"
#import "MCHelperToolClient.h"

#import <sys/sysctl.h>

#define _kAutomaticRefreshInterval 10.0

NSString* MCProcessMonitorChangeNotification = @"MCProcessMonitorChangeNotification";

@interface MCProcessMonitor(Private)
- (void)_sampleAll;
@end

@implementation MCProcessMonitor

- (id)init
{
	if( (self = [super init]) != nil )
	{
		mProcesses = [[NSMutableDictionary alloc] init];

		mHelperTool = [[MCHelperToolClient alloc] init];

		[[NSNotificationCenter defaultCenter]
			addObserver: self
			selector: @selector(_foregroundProcessChanged:)
			name: MCForegroundProcessChanged
			object: [MCForegroundProcessMonitor sharedWatcher]];

		mNonretainedTimer = [NSTimer scheduledTimerWithTimeInterval: _kAutomaticRefreshInterval
			target: self
			selector: @selector( _periodicSample: )
			userInfo: nil
			repeats: YES];
	
		[self _sampleAll];
	}
	
	return self;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver: self];
	[MCForegroundProcessMonitor terminateSharedWatcher];	

	[mNonretainedTimer invalidate];
	mNonretainedTimer = nil;

	[mProcesses release];
	mProcesses = nil;

	[mHelperTool release];
	mHelperTool = nil;

	[super dealloc];
}

#pragma mark -

- (pid_t)foregroundProcess
{
	ProcessSerialNumber psn;
	pid_t pid;
	
	if( GetFrontProcess( &psn ) != noErr )
		return 0;
		
	if( GetProcessPID( &psn, &pid ) != noErr )
		return 0;
	
	return pid;
}

- (const process_memory_stats*)statsForProcess: (pid_t)pid
{
	return [[mProcesses objectForKey: [NSNumber numberWithInt: pid]] bytes];
}

#pragma mark -

- (NSData*)_loadStatsForProcess: (pid_t)pid
{
	NSData* oldStats;
	NSData* newStats;
	
	oldStats = [mProcesses objectForKey: [NSNumber numberWithInt: pid]];

	newStats = [mHelperTool loadProcessMemoryStatsForPid: pid compareStats: oldStats];

	return newStats;
}

- (void)_sampleOne: (pid_t)pid
{
	NSData* statsData;
	NSNumber* key;
	
	if( !pid )
		return;
	
	key = [NSNumber numberWithInt: pid];

	if( (statsData = [self _loadStatsForProcess: pid]) == nil )
	{
		if( [mProcesses objectForKey: key] )
			[mProcesses removeObjectForKey: key];
		
		return;
	}
	
	[mProcesses setObject: statsData forKey: key];
	[[NSNotificationCenter defaultCenter] postNotificationName: MCProcessMonitorChangeNotification object: self];
}

- (void)_sampleAll
{
	NSMutableDictionary* updatedProcesses;
	ProcessSerialNumber psn = { 0, kNoProcess };
	pid_t pid;
	NSDictionary* procInfo;
	NSData* statsData;
		
	updatedProcesses = [[NSMutableDictionary alloc] initWithCapacity: [mProcesses count]];
	
	while( GetNextProcess( &psn ) == noErr )
	{
		if( GetProcessPID( &psn, &pid ) != noErr )
			continue;
		
		procInfo = [(NSDictionary*)ProcessInformationCopyDictionary( &psn, kProcessDictionaryIncludeAllInformationMask ) autorelease];
		if( !procInfo )
			continue;
		
		if( [[procInfo objectForKey: @"LSBackgroundOnly"] intValue] == 1 ||
			[[procInfo objectForKey: @"LSUIElement"] intValue] == 1 )
		{
			process_memory_stats newStats;
			memset( &newStats, 0, sizeof(newStats) );
			newStats.fPid = pid;
			newStats.fStatus = kProcessMemoryStatsNoData;
			statsData = [NSData dataWithBytes: &newStats length: sizeof(newStats)];
		}
		else
		{
			statsData = [self _loadStatsForProcess: pid];
		}
		
		if( statsData )
			[updatedProcesses setObject: statsData forKey: [NSNumber numberWithInt: pid]];
	}

	[mProcesses release];
	mProcesses = updatedProcesses;

	[[NSNotificationCenter defaultCenter] postNotificationName: MCProcessMonitorChangeNotification object: self];
}

- (void)_foregroundProcessChanged: (NSNotification*)note
{
	[self _sampleOne: [self foregroundProcess]];
}

- (void)_periodicSample: (NSTimer*)timer
{
	[self _sampleAll];
}


@end
